const { withProjectBuildGradle } = require("@expo/config-plugins");

module.exports = function withAndroidHolderSDK(config) {
  return withProjectBuildGradle(config, (config) => {
    const buildGradle = config.modResults.contents;
    const kotlinVersion = "1.9.0";
    const minSdkVersion = "24";

    // Split build.gradle into lines for easier manipulation
    const lines = buildGradle.split("\n");

    // Helper function to find a block within a specific parent block
    const findBlock = (parentBlockName, blockName) => {
      let startIndex = -1;
      let endIndex = -1;
      let inParentBlock = false;
      let braceCount = 0;

      for (let i = 0; i < lines.length; i++) {
        const line = lines[i].trim();

        // Detect parent block
        if (!inParentBlock) {
          if (
            parentBlockName === "" &&
            line.startsWith(blockName) &&
            line.includes("{")
          ) {
            inParentBlock = true;
            braceCount =
              (line.match(/{/g) || []).length - (line.match(/}/g) || []).length;
            startIndex = i;
            continue;
          }
          if (
            parentBlockName !== "" &&
            line.startsWith(parentBlockName) &&
            line.includes("{")
          ) {
            inParentBlock = true;
            braceCount =
              (line.match(/{/g) || []).length - (line.match(/}/g) || []).length;
          }
        } else {
          // Update brace count
          braceCount +=
            (line.match(/{/g) || []).length - (line.match(/}/g) || []).length;

          // Detect child block within parent block
          if (line.startsWith(blockName) && line.includes("{")) {
            startIndex = i;
            // Find the end of this block
            for (let j = i + 1; j < lines.length; j++) {
              braceCount +=
                (lines[j].match(/{/g) || []).length -
                (lines[j].match(/}/g) || []).length;
              if (braceCount === 0) {
                endIndex = j;
                break;
              }
            }
            break;
          }

          // Exit parent block
          if (braceCount === 0) {
            inParentBlock = false;
          }
        }
      }

      return { start: startIndex, end: endIndex };
    };

    // **1. Update ext block within buildscript**
    const extBlock = findBlock("buildscript", "ext");
    if (extBlock.start !== -1) {
      // Update existing ext block
      const extContent = lines.slice(extBlock.start + 1, extBlock.end);

      // Update or add kotlinVersion
      const kotlinLineIndex = extContent.findIndex((line) =>
        line.trim().startsWith("kotlinVersion")
      );
      if (kotlinLineIndex !== -1) {
        extContent[kotlinLineIndex] = `    kotlinVersion = "${kotlinVersion}"`;
      } else {
        extContent.push(`    kotlinVersion = "${kotlinVersion}"`);
      }

      // Update or add minSdkVersion
      const minSdkLineIndex = extContent.findIndex((line) =>
        line.trim().startsWith("minSdkVersion")
      );
      if (minSdkLineIndex !== -1) {
        extContent[minSdkLineIndex] = `    minSdkVersion = '${minSdkVersion}'`;
      } else {
        extContent.push(`    minSdkVersion = '${minSdkVersion}'`);
      }

      // Replace the old ext block with the updated content
      lines.splice(
        extBlock.start + 1,
        extBlock.end - extBlock.start - 1,
        ...extContent
      );
    } else {
      // If ext block doesn't exist, create it inside buildscript
      const buildscriptBlock = findBlock("", "buildscript");
      if (buildscriptBlock.start !== -1) {
        lines.splice(
          buildscriptBlock.start + 1,
          0,
          `    ext {`,
          `        kotlinVersion = "${kotlinVersion}"`,
          `        minSdkVersion = '${minSdkVersion}'`,
          `    }`
        );
      }
    }

    // **2. Add classpath dependencies within buildscript.dependencies**
    const dependenciesBlock = findBlock("buildscript", "dependencies");
    const classpathsToAdd = [
      `classpath("org.jetbrains.kotlin:kotlin-gradle-plugin:${kotlinVersion}")`,
      `classpath("org.jetbrains.kotlin:kotlin-android-extensions:${kotlinVersion}")`,
    ];

    for (const classpath of classpathsToAdd) {
      const exists = lines.some((line) => line.trim() === classpath);
      if (!exists && dependenciesBlock.start !== -1) {
        lines.splice(dependenciesBlock.start + 1, 0, `        ${classpath}`);
      }
    }

    // **3. Add required Maven repositories within allprojects.repositories**
    const allprojectsBlock = findBlock("", "allprojects");
    const repositoriesToAdd = [
      `maven { url = "$rootDir/../node_modules/@mattrglobal/mobile-credential-holder-react-native/android/frameworks" }`,
      `maven { url 'https://oss.sonatype.org/content/repositories/snapshots/' }`,
    ];

    if (allprojectsBlock.start !== -1) {
      const repositoriesBlock = findBlock("allprojects", "repositories");
      if (repositoriesBlock.start !== -1) {
        for (const repo of repositoriesToAdd) {
          const exists = lines
            .slice(repositoriesBlock.start + 1, repositoriesBlock.end)
            .some((line) => line.trim() === repo);
          if (!exists) {
            lines.splice(repositoriesBlock.start + 1, 0, `        ${repo}`);
          }
        }
      } else {
        // If repositories block doesn't exist inside allprojects, create it
        lines.splice(
          allprojectsBlock.start + 1,
          0,
          `    repositories {`,
          ...repositoriesToAdd.map((repo) => `        ${repo}`),
          `    }`
        );
      }
    } else {
      // If allprojects block doesn't exist, create it
      lines.push(`
  
  allprojects {
      repositories {
          ${repositoriesToAdd.map((repo) => repo).join("\n          ")}
      }
  }
  `);
    }

    // **4. Update the buildGradle content in config**
    config.modResults.contents = lines.join("\n");

    return config;
  });
};
